""" pynchon.testing

WARNING: this file exists so that some things can be shared across
the different test suites (units, integration, smoke-tests) which
are defined by this project.  To avoid more WETness in your
`{suite_name}/conftest.py`, lift common values up into this file.
"""
