

## {{header|default('Version Info')}}

This content is auto-generated, and simply points to commits at the time project-documentation was last refreshed.  Note that docs-refresh time/date/commit may not reflect when the package was released, since that can happen on a different schedule!

```
version: `{{python.package.version}}`
git_hash:`{{git.hash}}`
pynchon_version: `{{pynchon.version}}`
```
