

<table>
  <tr>
    <td colspan=2><strong>
    {{project.name}}
      </strong>&nbsp;&nbsp;&nbsp;&nbsp;
      <small><small>
      </small></small>
    </td>
  </tr>
  <tr>
    <td width=15%><img src=img/icon.png style="width:150px"></td>
    <td>
    {{project.name}}
    </td>
  </tr>
</table>
