""" pynchon.codemod.commands.docstrings
"""

from .base import base  # noqa

# from .javadoc import *  # noqa

# import simple # noqa
# from libcst.codemod import CodemodContext, VisitorBasedCodemodCommand

# from .javadoc import javadoc

# class klass(base):
#     DESCRIPTION: str = """\tAdds or updates javadoc for Classes"""
