
**entrypoint:** `python -m{{subcommand.invocation_sample|default(subcommand.dotpath)}}` [src]({{subcommand.src_url}})
{%if subcommand.click|default(false)%}[using click](#){%endif%}
<hr/>

{%if subcommand.help%}
```
{{subcommand.help}}
```
{%else%}
```
{{subcommand}}
```
{%endif%}

