""" pynchon.plugins.release
"""

# from pynchon import models
# from pynchon.util import lme, typing
#
# LOGGER = lme.get_logger(__name__)
#
#
# class Release(models.Tool):
#     """
#     Namespace for rendering docs-generation commands from other plugins
#     """
#
#     name = cli_name = 'gen'
#     priority = -1
#     config_class = None
#     cli_subsumes: typing.List[typing.Callable] = []
#
#     # @common.groop("api", parent=groups.gen)
#     # def gen_api() -> None:
#     #     """
#     #     Generate API docs from python modules, packages, etc
#     #     """
#     #
#     # @common.groop("cli", parent=groups.gen)
#     # def gen_cli():
#     #     """Generate CLI docs"""
#     #
